'''Simple data loader module.

Loads data files from the "data" directory shipped with a game.

Enhancing this to handle caching etc. is left as an exercise for the reader.

Note that pyglet users should probably just add the data directory to the
pyglet.resource search path.
'''

import os
import pygame

data_py = os.path.abspath(os.path.dirname(__file__))
data_dir = os.path.normpath(os.path.join(data_py, '.', 'data'))

def filepath(filename):
    '''Determine the path to a file in the data directory.
    '''
    return os.path.join(data_dir, filename)

def load(filename, mode='rb'):
    '''Open a file in the data directory.

    "mode" is passed as the second arg to open().
    '''
    return open(os.path.join(data_dir, filename), mode)

def load_image(filename,scale):
    '''Open an image in the data directory.'''
    img = pygame.image.load(filepath(filename)).convert()
    if scale == 2:
        return pygame.transform.scale2x(img)
    elif scale != 1:
        return pygame.transform.scale(img,(img.get_width()*scale,img.get_height()*scale))
    else:
        return img

def load_ck_image(filename,scale,position=(0,0)):
    '''Open a color keyed image in the data directory 
    
    the color at position (default (0,0)) is set as the color key'''
    img = load_image(filename,scale)
    img.set_colorkey(img.get_at(position))
    return img

