
import sys

import pygame
import pygame.locals as loc
import pygame.font

import data, game

class Character(pygame.sprite.Sprite):
    '''A character that walks on the screen'''

    def __init__(self,image,size,position,parent):
        ''''''
        pygame.sprite.Sprite.__init__(self)
        self.parent = parent
        self.images=[]
        for i in xrange(4):
            anim = []
            for j in xrange(3):
                anim.append(image.subsurface(pygame.Rect(size*j,size*i,size,size)))
            anim.append(image.subsurface(pygame.Rect(size,size*i,size,size)))
            self.images.append(anim)

        self.frame = 1
        self.facing = 2
        self.moving = False
        self.target = (0,0)
        self.direction = -1
        #self.callback = None
        self.cb_opt = None
        self.image = self.images[2][1]
        self.rect = self.image.get_rect()
        self.rect.topleft = position

    def update(self):
        ''''''
        if self.moving:
            if self.rect.topleft == self.target:
                self.moving = False
                self.callback(self.cb_opt)
            else:
                if self.rect.left < self.target[0]:
                    self.rect.left += 4 * self.parent.conf.scale
                    self.facing = 1
                elif self.rect.left > self.target[0]:
                    self.rect.left -= 4 * self.parent.conf.scale
                    self.facing = 3
                elif self.rect.top < self.target[1]:
                    self.rect.top += 4 * self.parent.conf.scale
                    self.facing = 2
                elif self.rect.top > self.target[1]:
                    self.rect.top -= 4 * self.parent.conf.scale
                    self.facing = 0
            if self.parent.game.turns % 4 == 0:
                self.frame = (self.frame + 1) % 4
            self.image = self.images[self.facing][self.frame]

    def clicked(self):
        '''Possibly do something stupid'''

        print "Ehi! don't click on me!"

    def call(self,position,callback,p_type):
        '''Move the character to position.

        When position has been reached, call the callback function'''

        self.target = position
        self.moving = True

        #TODO: actually walk :D
        self.callback = callback
        self.cb_opt = p_type

    def satisfy(self,p_type):
        '''Empty method'''


class Request(pygame.sprite.Sprite):
    '''A request for some kind of potion'''

    def __init__(self,image,size,position,p_type,slot,parent):
        ''''''
        pygame.sprite.Sprite.__init__(self)
        self.parent = parent
        self.score = 80
        self.slot = slot
        self.p_type = p_type
        self.image = image.subsurface(pygame.Rect(size * p_type,0,size,size))
        self.rect = self.image.get_rect()
        self.rect.topleft = position

    def update(self):
        ''''''
        if self.score > 0:
            self.score -= 1

    def clicked(self):
        ''''''
        if self.parent.game.player.potion == self.p_type:
            self.parent.player.call(
                    (self.rect.left,
                        self.parent.sizes['tile_0'][1] + self.parent.sizes['tile'] * 3),
                    self.satisfy,
                    self.p_type)

    def satisfy(self,p_type):
        self.parent.game.score += self.score
        self.parent.game.player.potion = None
        self.parent.request_slots[self.slot] = False
        self.parent.game.requests -= 1
        self.parent.sprites.remove(self)


class Ingredients(pygame.sprite.Sprite):
    '''An ingredients holder'''

    def __init__(self,image,size,position,p_type,parent):
        pygame.sprite.Sprite.__init__(self)
        self.parent = parent
        self.p_type = p_type
        self.image = image.subsurface(pygame.Rect(size*p_type,0,size,size))
        self.rect = self.image.get_rect()
        self.rect.topleft = position

    def clicked(self):
        self.parent.player.call(
                    (self.rect.left,
                        self.parent.sizes['tile_0'][1] + self.parent.sizes['tile'] * 1.5),
                self.parent.game.player.grab_ingredients,
                self.p_type)

    def satisfy(self,p_type):
        '''Empty method'''


class Alembic(pygame.sprite.Sprite):
    '''An alembic'''

    def __init__(self,image,width,height,position,parent):
        ''''''
        pygame.sprite.Sprite.__init__(self)
        self.parent = parent
        self.full = 0
        self.p_type = 0
        self.images = []
        for i in xrange(3):
            anim = []
            for j in xrange(11):
                anim.append(image.subsurface(pygame.Rect(width*j,height*i,width,height)))
            self.images.append(anim)
        self.image = self.images[0][0]
        self.rect = self.image.get_rect()
        self.rect.topleft = position

    def clicked(self):
        if self.full == 0 and self.parent.game.player.ingredients != None:
            self.parent.player.call(
                    (self.rect.left,
                        self.parent.sizes['tile_0'][1] + self.parent.sizes['tile'] * 1.5),
                    self.fill,
                    self.parent.game.player.ingredients)
        elif self.full == 10 and \
                self.parent.game.player.ingredients == None and \
                self.parent.game.player.potion == None:
            self.parent.player.call(
                    (self.rect.left,
                        self.parent.sizes['tile_0'][1] + self.parent.sizes['tile'] * 1.5),
                    self.empty,
                    self.p_type)


    def update(self):
        if self.full > 0 and self.full < 10 and self.parent.game.turns % 4 == 0:
            self.full += 1
            self.image = self.images[self.p_type][self.full]

    def fill(self,p_type):
        self.full = 1
        self.p_type = p_type
        self.image = self.images[p_type][self.full]
        self.parent.game.player.ingredients = None

    def empty(self,p_type):
        self.full = 0
        self.parent.game.player.grab_potion(self.p_type)
        self.image = self.images[self.p_type][self.full]

    def satisfy(self,p_type):
        '''Empty the alembic (at the end of the day'''
        self.full = 0
        self.image = self.images[self.p_type][self.full]


class UI:
    def __init__(self,conf):
        '''create and initialize a pygame interface'''

        self.conf = conf

        # Set sizes and positions based on the configuration options

        self.sizes = {}
        if self.conf.wide:
            self.sizes['offset'] = 40 * self.conf.scale
            self.sizes['screen'] = (400 * self.conf.scale , 240 * self.conf.scale)
            self.sizes['tile_0'] = (72,24)
        else:
            self.sizes['offset'] = 0
            self.sizes['screen'] = (320 * self.conf.scale , 240 * self.conf.scale)
            self.sizes['tile_0'] = (32 * self.conf.scale ,24 * self.conf.scale)
        self.sizes['tile'] = 32 * self.conf.scale

        # Initialize pygame

        pygame.init()
        pygame.font.init()

        pygame.event.set_allowed(None)
        pygame.event.set_allowed([loc.KEYUP,loc.MOUSEBUTTONUP,loc.QUIT])

        if self.conf.fullscreen:
            flags = loc.FULLSCREEN
        else:
            flags = 0
        self.screen = pygame.display.set_mode(self.sizes['screen'],flags)
        pygame.display.set_caption('Alembic Flipper')

        if self.conf.wide:
            self.background = data.load_image('bg-wide.png',self.conf.scale)
        else:
            self.background = data.load_image('bg.png',self.conf.scale)
        self.screen.blit(self.background,(0,0))

        pygame.display.flip()

        self.clock = pygame.time.Clock()

        # Sprite management

        self.sprites = pygame.sprite.RenderUpdates()

        self.player = Character(
                data.load_ck_image('alembic_flipper.png',self.conf.scale),
                32*self.conf.scale,
                (self.sizes['tile_0'][0] + self.sizes['tile'] * 2, 
                    self.sizes['tile_0'][1] + self.sizes['tile'] * 3),
                self)
        self.sprites.add(self.player)

        self.alembic_image = data.load_ck_image('alembic.png',self.conf.scale)
        for i in xrange(3):
            self.sprites.add(
                    Alembic(
                        self.alembic_image,
                        32*self.conf.scale,24*self.conf.scale,
                        ((176 + 32 * i)*self.conf.scale + self.sizes['offset'], 40* self.conf.scale),
                        self))


        self.ingredients_image = data.load_image('ingredients.png',self.conf.scale)
        for i in xrange(3):
            self.sprites.add(
                    Ingredients(
                        self.ingredients_image,
                        32 * self.conf.scale,
                        ((48 + i*32) * self.conf.scale + self.sizes['offset'], 32 * self.conf.scale),
                        i,
                        self
                        ))

        pygame.display.update(self.sprites.draw(self.screen))

        # Images for sprites that will be created later in the game
        self.request_image = data.load_ck_image('requests.png',self.conf.scale)

        self.request_slots = [False] * 15

        # Create the game

        self.game = game.Game()

    def run(self):
        '''Run the interface'''

        self.running = True
        while self.running:
            # Check for events
            for event in pygame.event.get():
                if event.type == loc.KEYUP and event.key == loc.K_ESCAPE:
                    self.running = False
                elif event.type == loc.QUIT:
                    self.running = False
                elif event.type == loc.MOUSEBUTTONUP:
                    for sprite in self.sprites:
                        if sprite.rect.collidepoint(event.pos):
                            sprite.clicked()
                else:
                    print event
            # Check for day change or gameover
            self.game.turns += 1
            if self.game.requests >= 15:
                self.gameover()
                break
            elif self.game.turns >= 1200:
                if self.game.day >= 9:
                    self.endgame()
                    break
                else:
                    self.new_day()
            # Possibly create potions
            p_type = self.game.spawn_potion()
            if p_type != None:
                self.add_request(p_type)
            # Update sprites
            self.sprites.clear(self.screen,self.background)
            self.sprites.update()
            pygame.display.update(self.sprites.draw(self.screen))
            self.clock.tick(12)

    def backstory(self):
        '''Backstory introduction'''

        self.backstory = True
        if self.backstory:
            self.print_dialog(["Hello, you must be the new guy.",
                "Welcome to this potion drive-trough."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["Your job here will be simple",
                "..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["just get the ingredients for",
                "some potion..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["put them in one of the alembics,",
                "where they will boil nine times..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["and then bring the potion",
                "to its request..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["where it will be magically teleported",
                "to our customer's carriage."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["You don't even have to smile.",
                ""])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["Just try not to let too many",
                "customers outside wait."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["You will just have to brew the three",
                "most common kinds of potions"])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(['"Love", Cure and Awakening',
                "..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["As a campus student you surely",
                "know them already."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["At the end of the night shift",
                "I will be here to take your place."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["And after the ninth night",
                "you will get your salary"])
            self.wait_for_click()
        dest = self.print_dialog(["Good luck.",
            ""])
        self.wait_for_click()
        self.screen.blit(self.background,(0,0))
        pygame.display.update(dest)

    def gameover(self):
        '''Failure'''
        self.backstory = True
        if self.backstory:
            self.print_dialog(["Ehi! I can see the queue",
                "from my house"])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["Don't you think you should have",
                "served the customers a bit faster?"])
            self.wait_for_click()
        dest = self.print_dialog(["YOU'RE  FIRED!!!",
            ""])
        self.wait_for_click()
        self.screen.blit(self.background,(0,0))
        pygame.display.update(dest)
        self.running = False

    def new_day(self):
        '''Day change graphics'''
        for sprite in self.sprites:
            sprite.score = -10
            sprite.satisfy(0)

        self.backstory = True
        if self.backstory:
            self.print_dialog(["Good morning.",
                "Time for some rest."])
            self.wait_for_click()
        self.game.new_day()
        self.sprites.clear(self.screen,self.background)
        self.sprites.update()
        pygame.display.update(self.sprites.draw(self.screen))
        if self.backstory:
            self.print_dialog(["...",
                ""])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["...",
                "..."])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["Welcome back.",
                ""])
            self.wait_for_click()
        dest = self.print_dialog(["Good night,",
            "see you next morning."])
        self.wait_for_click()
        self.screen.blit(self.background,(0,0))
        pygame.display.update(dest)

    def endgame(self):
        '''Success'''
        self.backstory = True
        if self.backstory:
            self.print_dialog(["Well done",
                ""])
            self.wait_for_click()
        if self.backstory:
            self.print_dialog(["...",
                ""])
            self.wait_for_click()
        dest = self.print_dialog(["Here are your "+str(self.game.score)+" coppers.",
            ""])
        self.wait_for_click()
        self.screen.blit(self.background,(0,0))
        pygame.display.update(dest)
        self.running = False

    def add_request(self,p_type):
        '''Add graphics for a request in the first available space'''

        i = self.request_slots.index(False)
        
        self.sprites.add(Request(self.request_image,
                        16 * self.conf.scale,
                        ( (40 + i*16) * self.conf.scale + self.sizes['offset'],
                            152*self.conf.scale),
                        p_type,
                        i,
                        self))

        self.request_slots[i] = True

    def print_dialog(self,lines):
        ''''''

        self.font = pygame.font.SysFont('freesans',14*self.conf.scale)

        dialog = data.load_ck_image('dialog.png',self.conf.scale)

        dialog.blit(self.font.render(lines[0],True,(255,255,255)),(12*self.conf.scale,6*self.conf.scale))
        dialog.blit(self.font.render(lines[1],True,(255,255,255)),(12*self.conf.scale,22*self.conf.scale))

        dest = self.screen.blit(dialog,(32*self.conf.scale + self.sizes['offset'], 188*self.conf.scale))
        pygame.display.update(dest)
        return dest

    def wait_for_click(self):
        wait = True
        while wait:
            for event in pygame.event.get():
                if event.type == loc.KEYUP and event.key == loc.K_ESCAPE:
                    sys.exit(0)
                elif event.type == loc.QUIT:
                    sys.exit(0)
                elif event.type == loc.KEYUP and event.key == loc.K_SPACE:
                    self.backstory = False
                    wait = False
                elif event.type == loc.MOUSEBUTTONUP:
                    wait = False
            self.clock.tick(12)

